import java.util.*;
import java.lang.*;
import java.io.*;

/* Una ADT ParseTreeADT<A,B> per ParseTreeAPI<A,B> presentata il 25/5/2016
*/
/* (punto b) */
class ParseTree<A,B extends A> implements ParseTreeAPI<A,B>{
/* (punto c) */
    //Stato Concreto
    private A label;
    private Vector<ParseTree<A,B>> sons;
/* (punto d) */
    //AF(c)=[] se c.label==null
    //AF(c)=[a-(t1,...,tk) se (a==c.label)&&(k==c.sons.size()-1)&&
    //                        ti=c.sons.get(i+1) (per i\in[0..c.sons.size()-1])
    //I(c)=(if c.sons!=null then c.label!=null)
/* (punto e) */
//costruttori
    public ParseTree(){
        super();
    }
    public ParseTree(A label){
        this.label = label;
    }
    public ParseTree(B label, LinkedList<ParseTree<A,B>> L){
        this.label = label;
        sons = new Vector<ParseTree<A,B>>();
        for (ParseTree<A,B> t: L){
            sons.add(t);
        }
    }
    public boolean isEmpty(){
        return (label==null);
    }
    public boolean isLeaf(){
        if (label==null) return false;
        return ((label!=null)&&(sons.size()==0));
    }
    public boolean isRoot(){
        if (label==null) return false;
        return ((label!=null)&&(sons.size()!=0));
    }
    public A getLabel() throws invalidOperationException{
        if (label==null) throw new invalidOperationException();
        return label;
    }
    public LinkedList<ParseTreeAPI<A,B>> getSons() throws invalidOperationException{
        if (!isRoot()) throw new invalidOperationException();
        LinkedList<ParseTreeAPI<A,B>> res = new LinkedList<ParseTreeAPI<A,B>>();
        for(int i=0;i<sons.size();i++){
            res.add(i,sons.get(i));
            }
        return res;
    }
    public void set(Path p, ParseTreeAPI<A,B> t) throws invalidArgException{
        // Solleva invalidArgException se (p==null)||(t==null) o
        // p non \`e un cammino per this oppure t non \`e un ParseTree<A,B>
        // Modifica this:
        //   this.post==this.pre se p=0
        //   this.post=[a-(t1,...,tn1-1,T,tn1+1,...,tk) se:
        //      (1) p=n1.q
        //      (2) this.pre=[a-(t1,...,tn1-1,tn1,tn1+1,...,tk)
        //      (3) tn1 \`e modificata in T dopo tn1.set(q,t).
        if ((p==null)||(t==null)) throw new invalidArgException("set: p o t sono null");
        if (! (t instanceof ParseTree)) throw new invalidArgException("set: t non di tipo atteso");
        if (p.isEmpty()) return;
        int n1 = 1; Path q = null;
        try{n1 = p.next();q = p.rest();}
        catch(invalidOperationException e){throw new invalidArgException("set: operazioni next o rest su p non valide");}
        if (n1>sons.size()) throw new invalidArgException("set: p non è un cammino per t");
        if (q.isEmpty()){sons.set(n1-1,(ParseTree<A,B>)t); return;}
        ((ParseTree<A,B>)sons.get(n1-1)).set(q,t);
    }
    //ADDITIONAL(equals,clone,elements,toString)
    //equals ereditata perchè mutable
    public ParseTree<A,B> clone() throws CloneNotSupportedException{
        ParseTree<A,B> res = (ParseTree<A,B>) super.clone();
        Vector<ParseTree<A,B>> sonsCopy = (Vector<ParseTree<A,B>>)sons.clone();
        res.sons = sonsCopy;
        return res;
    }
    public LinkedList<ParseTree<A,B>> elements(){
        // Calcola lista di tutti i sotto-alberi di this (this incluso)
        LinkedList<ParseTree<A,B>> res = new LinkedList<ParseTree<A,B>>();
        res.add(this);
        if (isEmpty()||isLeaf()) return res;
        for(ParseTree<A,B> t: sons){
            LinkedList<ParseTree<A,B>> telm = t.elements();
            for(ParseTree<A,B> tt: t.sons){
                res.add(tt);
            }
        }
        return res;
    }
    public String toString(){
        String res = "[";
        if (isEmpty()) return (res+"]");
        res=res+label.toString();
        if (isLeaf()) return (res+"]");
        res=res+"-("+sons.get(0).toString();
        for(int i=1;i<sons.size();i++){
            res=res+","+sons.get(i).toString();
        }
        return (res+")]");
    }
}
